<?php

namespace Lightweb_Media\UPD_Amazon\Admin;

defined( 'ABSPATH' ) || exit;

use Lightweb_Media\UPD_Amazon\Abstracts\UPD_Amazon_Singleton_Abstract;
use Lightweb_Media\UPD_Amazon\Admin\Interfaces\UPD_Amazon_Admin_AJAX_Interface;
use Lightweb_Media\UPD_Amazon\UPD_Amazon_Cron_Jobs;
use Lightweb_Media\UPD_Amazon\UPD_Amazon_Search_Replacer;

/**
 * Class UPD_Amazon_Admin_AJAX
 *
 * @package Lightweb_Media\UPD_Amazon\Admin
 */
class UPD_Amazon_Admin_AJAX extends UPD_Amazon_Singleton_Abstract implements UPD_Amazon_Admin_AJAX_Interface {

	/**
	 * UPD_Amazon_Admin_AJAX constructor
	 */
	public function __construct() {
		if ( is_admin() ) {
			$this->init_hooks();
		}
	}

	/**
	 * Init / set up hooks
	 */
	private function init_hooks(): void {
		add_action( 'wp_ajax_upd_amazon_expand', array( $this, 'wp_ajax_upd_amazon_expand_action' ) );
		add_action( 'wp_ajax_upd_amazon_update', array( $this, 'wp_ajax_upd_amazon_update_action' ) );
		add_action( 'wp_ajax_upd_amazon_no_follow', array( $this, 'wp_ajax_upd_amazon_no_follow_action' ) );
		add_action( 'wp_ajax_upd_amazon_no_affiliate', array( $this, 'wp_ajax_upd_amazon_no_affiliate_action' ) );
		add_action( 'wp_ajax_upd_amazon_get_progress', array( $this, 'wp_ajax_upd_amazon_get_progress_action' ) );
	}

	/**
	 * Schedules the expanding of shortened Amazon links
	 *
	 * @return void
	 */
	public function wp_ajax_upd_amazon_expand_action(): void {
		check_ajax_referer( 'upd-amazon', 'security' );

		$args = array(
			'posts_idx' => 0
		);

		$this->check_schedule_clearance( 'upd_amazon_expand', $args );

		UPD_Amazon_Cron_Jobs::get_instance()->run_cron( 'upd_amazon_expand', $args );

		$this->send_scheduled_response( 'upd_amazon_expand', $args );
	}

	/**
	 * Schedules the update from short to long links including the affiliate tag
	 *
	 * @return void
	 */
	public function wp_ajax_upd_amazon_update_action(): void {
		check_ajax_referer( 'upd-amazon', 'security' );

		$new_id = sanitize_text_field( $_POST['new_id'] );

		if ( empty( $new_id ) ) {
			$response = array(
				'message' => 'Please enter a correct tag.',
			);
			wp_send_json_error( $response, 400 );

			/** @noinspection ForgottenDebugOutputInspection */
			wp_die();
		}

		$args = array(
			'new_id'    => $new_id,
			'posts_idx' => 0
		);

		$this->check_schedule_clearance( 'upd_amazon_update', $args );

		UPD_Amazon_Cron_Jobs::get_instance()->run_cron( 'upd_amazon_update', $args );

		$this->send_scheduled_response( 'upd_amazon_update', $args );
	}

	/**
	 * Schedules the adding of the no follow attribute to the links
	 *
	 * @return void
	 */
	public function wp_ajax_upd_amazon_no_follow_action(): void {
		check_ajax_referer( 'upd-amazon', 'security' );

		$args = array(
			'posts_idx' => 0
		);

		$this->check_schedule_clearance( 'upd_amazon_no_follow', $args );

		UPD_Amazon_Cron_Jobs::get_instance()->run_cron( 'upd_amazon_no_follow', $args );

		$this->send_scheduled_response( 'upd_amazon_no_follow', $args );
	}

	/**
	 * Schedules the removing of the Amazon affiliate links
	 *
	 * @return void
	 */
	public function wp_ajax_upd_amazon_no_affiliate_action(): void {
		check_ajax_referer( 'upd-amazon', 'security' );

		$args = array(
			'posts_idx' => 0
		);

		$this->check_schedule_clearance( 'upd_amazon_no_affiliate', $args );

		UPD_Amazon_Cron_Jobs::get_instance()->run_cron( 'upd_amazon_no_affiliate', $args );

		$this->send_scheduled_response( 'upd_amazon_no_affiliate', $args );
	}

	/**
	 * Returns the current progress of a job/action
	 *
	 * @return void
	 */
	public function wp_ajax_upd_amazon_get_progress_action(): void {
		check_ajax_referer( 'upd-amazon', 'security' );

		$job = UPD_Amazon_Search_Replacer::get_instance()->get_last_job( true );

		wp_send_json_success( $job );

		/** @noinspection ForgottenDebugOutputInspection */
		wp_die();
	}

	/**
	 * Send scheduled response
	 *
	 * @param string $hook
	 * @param array $args
	 *
	 * @return void
	 */
	private function send_scheduled_response( string $hook, array $args = array() ): void {
		$is_scheduled = UPD_Amazon_Cron_Jobs::get_instance()->is_scheduled( $hook, $args );

		if ( $is_scheduled ) {
			$response = array(
				'message' => 'Successfully initialized event. Please be patient, the process will start within a few seconds.',
			);
			wp_send_json_success( $response );
		} else {
			$response = array(
				'message' => 'Unable to schedule event. Please check, if WordPress can handle cron jobs.',
			);
			wp_send_json_error( $response, 500 );
		}

		/** @noinspection ForgottenDebugOutputInspection */
		wp_die();
	}

	/**
	 * Check if we have a job running at the moment to ensure that the job is not triggered twice
	 *
	 * @param string $hook
	 * @param array $args
	 *
	 * @return void
	 */
	private function check_schedule_clearance( string $hook, array $args = array() ): void {
		$is_scheduled = UPD_Amazon_Cron_Jobs::get_instance()->is_scheduled( $hook, $args );

		if ( upd_amazon_memory_time_limit_reached() ) {
			$response = array(
				'message' => 'You\'re servers available memory or time limit is too low to start the function. Please increase your memory or time limit.',
			);
			wp_send_json_error( $response, 500 );

			/** @noinspection ForgottenDebugOutputInspection */
			wp_die();
		}

		if ( $is_scheduled ) {
			$response = array(
				'message' => 'Can\'t re-schedule already started event.',
			);
			wp_send_json_error( $response, 500 );

			/** @noinspection ForgottenDebugOutputInspection */
			wp_die();
		}
	}
}
