<?php

namespace Lightweb_Media\UPD_Amazon;

defined( 'ABSPATH' ) || exit;

use Lightweb_Media\UPD_Amazon\Abstracts\UPD_Amazon_Singleton_Abstract;
use Lightweb_Media\UPD_Amazon\Interfaces\UPD_Amazon_Cron_Jobs_Interface;

/**
 * Class UPD_Amazon_Cron_Jobs
 *
 * @package Lightweb_Media\UPD_Amazon
 */
class UPD_Amazon_Cron_Jobs extends UPD_Amazon_Singleton_Abstract implements UPD_Amazon_Cron_Jobs_Interface {

	/**
	 * UPD_Amazon_Cron_Jobs constructor
	 */
	public function __construct() {
		$this->init_hooks();
	}

	/**
	 * Init / set up hooks
	 */
	private function init_hooks(): void {
		add_action( 'upd_amazon_expand', array( $this, 'upd_amazon_expand_action' ) );
		add_action( 'upd_amazon_update', array( $this, 'upd_amazon_update_action' ), 10, 2 );
		add_action( 'upd_amazon_no_follow', array( $this, 'upd_amazon_no_follow_action' ) );
		add_action( 'upd_amazon_no_affiliate', array( $this, 'upd_amazon_no_affiliate_action' ) );
	}

	/**
	 * Runs scheduled expanding of shortened Amazon links
	 *
	 * @param int $posts_idx
	 */
	public function upd_amazon_expand_action( int $posts_idx ): void {
		UPD_Amazon_Search_Replacer::get_instance()->expand( $posts_idx );
	}

	/**
	 * Runs scheduled update from short to long links including the affiliate tag
	 *
	 * @param string $new_id
	 * @param int $posts_idx
	 */
	public function upd_amazon_update_action( string $new_id, int $posts_idx ): void {
		UPD_Amazon_Search_Replacer::get_instance()->update_db( $new_id, $posts_idx );
	}

	/**
	 * Runs scheduled adding of the no follow attribute to the links
	 *
	 * @param int $posts_idx
	 */
	public function upd_amazon_no_follow_action( int $posts_idx ): void {
		UPD_Amazon_Search_Replacer::get_instance()->no_follow( $posts_idx );
	}

	/**
	 * Runs scheduled removing of the Amazon affiliate links
	 *
	 * @param int $posts_idx
	 */
	public function upd_amazon_no_affiliate_action( int $posts_idx ): void {
		UPD_Amazon_Search_Replacer::get_instance()->no_affiliate( $posts_idx );
	}

	/**
	 * Runs a cron immediately by skipping any schedule
	 *
	 * @param string $cron
	 * @param array $args
	 */
	public function run_cron( string $cron, array $args = array() ): void {
		if ( empty( $cron ) ) {
			/** @noinspection ForgottenDebugOutputInspection */
			wp_die( 'Action failed. Please refresh the page and retry.' );
		}

		delete_transient( 'doing_cron' );
		$this->schedule_single( time(), $cron, false, $args );
		spawn_cron();
	}

	/**
	 * Schedules a single event
	 *
	 * @param int $timestamp
	 * @param string $hook
	 * @param bool $check_scheduled
	 * @param array $args
	 */
	public function schedule_single( int $timestamp, string $hook, bool $check_scheduled = true, array $args = array() ): void {
		if ( $check_scheduled && $this->is_scheduled( $hook, $args ) ) {
			return;
		}

		wp_schedule_single_event( $timestamp, $hook, $args );
	}

	/**
	 * Schedules a recurring event
	 *
	 * @param int $timestamp
	 * @param string $recurrence
	 * @param string $hook
	 * @param bool $check_scheduled
	 * @param array $args
	 */
	public function schedule_recurring( int $timestamp, string $recurrence, string $hook, bool $check_scheduled = true, array $args = array() ): void {
		if ( $check_scheduled && $this->is_scheduled( $hook, $args ) ) {
			return;
		}

		wp_schedule_event( $timestamp, $recurrence, $hook, $args );
	}

	/**
	 * Clears scheduled hook
	 *
	 * @param string $hook
	 *
	 * @return void
	 */
	public function clear( string $hook ): void {
		wp_unschedule_hook( $hook );
	}

	/**
	 * Check if an event is already scheduled
	 *
	 * @param string $hook
	 * @param array $args
	 *
	 * @return bool
	 */
	public function is_scheduled( string $hook, array $args = array() ): bool {
		if ( ! wp_next_scheduled( $hook, $args ) ) {
			return false;
		}

		return true;
	}
}
