<?php

namespace Lightweb_Media\UPD_Amazon;

defined( 'ABSPATH' ) || exit;

use Automattic\Jetpack\Constants;
use Lightweb_Media\UPD_Amazon\Abstracts\UPD_Amazon_Singleton_Abstract;
use Lightweb_Media\UPD_Amazon\Interfaces\UPD_Amazon_Install_Interface;
use wpdb;

/**
 * Class UPD_Amazon_Install
 *
 * @package Lightweb_Media\UPD_Amazon
 */
class UPD_Amazon_Install extends UPD_Amazon_Singleton_Abstract implements UPD_Amazon_Install_Interface {

	/**
	 * Jobs table identifier
	 *
	 * @var string
	 */
	public const JOBS_TABLE_NAME = 'upd_amazon_jobs';

	/**
	 * WordPress database
	 *
	 * @var wpdb
	 */
	private wpdb $wpdb;

	/**
	 * Current version
	 *
	 * @var string
	 */
	private string $current_upd_amazon_version;

	/**
	 * UPD_Amazon_Install constructor
	 */
	public function __construct() {
		global $wpdb; //DB

		$this->wpdb                       = $wpdb;
		$this->current_upd_amazon_version = get_option( 'upd_amazon_version' );

		$this->init_hooks();
	}

	/**
	 * Init / set up hooks
	 */
	private function init_hooks(): void {
		add_action( 'admin_init', array( $this, 'admin_init_action' ) );
	}

	/**
	 * Install
	 */
	public function install(): void {
		if ( ! is_blog_installed() ) {
			return;
		}

		$this->create_update_db_tables();
		$this->update_upd_amazon_version();
	}

	/**
	 * Check for plugin update -> install again to verify data
	 */
	public function admin_init_action(): void {
		if ( ! defined( 'IFRAME_REQUEST' ) && version_compare( $this->current_upd_amazon_version, Constants::get_constant( 'UPD_AMAZON_VERSION' ), '<' ) ) {
			$this->install();
		}
	}

	/**
	 * Set up the database tables which the plugin needs to function
	 * This is called from `install` method and is executed in-sync when the plugin is installed or updated. This can also be called optionally from `verify_base_tables`
	 */
	private function create_update_db_tables(): void {
		if ( ! function_exists( 'dbDelta' ) ) {
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		}

		$this->wpdb->hide_errors();

		dbDelta( $this->get_db_schema() );
	}

	/**
	 * Update plugin version
	 */
	private function update_upd_amazon_version(): void {
		delete_option( 'upd_amazon_version' );
		update_option( 'upd_amazon_version', Constants::get_constant( 'UPD_AMAZON_VERSION' ) );
	}

	/**
	 * Get DB table schema
	 *
	 * @return string
	 */
	private function get_db_schema(): string {
		$collate = '';
		$prefix  = $this->wpdb->prefix;

		if ( $this->wpdb->has_cap( 'collation' ) ) {
			$collate = $this->wpdb->get_charset_collate();
		}

		$jobs_table_name   = $prefix . self::JOBS_TABLE_NAME;

		return "
		CREATE TABLE {$jobs_table_name} (
		    id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
		    started_at DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
		    finished_at DATETIME DEFAULT NULL,
			keyword text NOT NULL,
			percent INTEGER NOT NULL,
			processed integer NULL default 0,
			PRIMARY KEY (id) 
		) $collate;
		";
	}
}
