<?php

namespace Lightweb_Media\UPD_Amazon;

defined( 'ABSPATH' ) || exit;

use DOMDocument;
use Lightweb_Media\UPD_Amazon\Abstracts\UPD_Amazon_Singleton_Abstract;
use Lightweb_Media\UPD_Amazon\Interfaces\UPD_Amazon_Search_Replacer_Interface;
use wpdb;

/**
 * Class UPD_Amazon_Search_Replacer
 *
 * @package Lightweb_Media\UPD_Amazon
 */
class UPD_Amazon_Search_Replacer extends UPD_Amazon_Singleton_Abstract implements UPD_Amazon_Search_Replacer_Interface {

	/**
	 * WordPress database
	 *
	 * @var wpdb
	 */
	private wpdb $wpdb;

	/**
	 * UPD_Amazon_Search_Replacer constructor
	 */
	public function __construct() {
		global $wpdb; //DB

		$this->wpdb = $wpdb;
	}

	/**
	 * Performs the expanding of shortened Amazon links
	 *
	 * @param int $posts_idx
	 *
	 * @return void
	 */
	public function expand( int $posts_idx ): void {
		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			remove_action( 'post_updated', 'wp_save_post_revision' );
		}

		if ( $posts_idx === 0 && $this->add_job() < 0 ) {
			return;
		}

		$posts         = $this->get_posts();
		$posts_len     = count( $posts );
		$prev_progress = 1;
		$processed     = 0;

		foreach ( $posts as $index => $post ) {
			if ( $posts_idx > 0 && $index < $posts_idx ) { // Skip already processed items
				continue;
			}

			if ( upd_amazon_memory_time_limit_reached() ) {
				UPD_Amazon_Cron_Jobs::get_instance()->clear( 'upd_amazon_expand' );

				UPD_Amazon_Cron_Jobs::get_instance()->schedule_single( time() + 30, 'upd_amazon_expand', false, array(
					'posts_idx' => $index,
				) );

				return;
			}

			if ( ( $post->post_content !== "" ) && count( $this->find_links( $post->post_content ) ) > 0 ) {
				$new_content = preg_replace_callback( '/"(https?):\/\/(www\.)?amzn.to([^"]*)"|"(https?):\/\/(www\.)?amazon.([^"]*)"/', function ( $url ) {
					$newLink = $this->transform_link( str_replace( '"', '', $url[0] ), '', true );

					return '"' . $newLink . '"';
				}, $post->post_content );

				$post = array( 'ID' => $post->ID, 'post_content' => $new_content );
				wp_update_post( $post );

				$processed = $index + 1;
			}

			$current_progress = (int) ( ( $processed / $posts_len ) * 100 );

			if ( $current_progress >= $prev_progress + 5 ) {
				$this->set_progress( $current_progress );

				$prev_progress = $current_progress;
			}
		}

		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			add_action( 'post_updated', 'wp_save_post_revision' );
		}

		$this->set_progress( 100, $processed );
	}

	/**
	 * Performs the update from short to long links including the affiliate tag
	 *
	 * @param string $new_id
	 * @param int $posts_idx
	 *
	 * @return void
	 */
	public function update_db( string $new_id, int $posts_idx ): void {
		if ( ! defined( 'DISALLOW_UNFILTERED_HTML' ) ) {
			define( 'DISALLOW_UNFILTERED_HTML', false );
		}

		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			remove_action( 'post_updated', 'wp_save_post_revision' );
		}

		if ( $posts_idx === 0 && $this->add_job( $new_id ) < 0 ) {
			return;
		}

		$posts         = $this->get_posts();
		$posts_len     = count( $posts );
		$prev_progress = 1;
		$processed     = 0;

		foreach ( $posts as $index => $post ) {
			if ( $posts_idx > 0 && $index < $posts_idx ) { // Skip already processed items
				continue;
			}

			if ( upd_amazon_memory_time_limit_reached() ) {
				UPD_Amazon_Cron_Jobs::get_instance()->clear( 'upd_amazon_update' );

				UPD_Amazon_Cron_Jobs::get_instance()->schedule_single( time() + 30, 'upd_amazon_update', false, array(
					'new_id'     => $new_id,
					'posts_idx' => $index,
				) );

				return;
			}

			if ( ( $post->post_content !== "" ) && count( $this->find_links( $post->post_content ) ) > 0 ) {
				$new_content = preg_replace_callback( '/"(https?):\/\/(www\.)?amzn.to([^"]*)"|"(https?):\/\/(www\.)?amazon.([^"]*)"/', function ( $url ) use ( $new_id ) {
					$newLink = $this->transform_link( str_replace( '"', '', $url[0] ), $new_id );

					return '"' . $newLink . '"';
				}, $post->post_content );

				$post = array( 'ID' => $post->ID, 'post_content' => $new_content );
				wp_update_post( wp_slash( $post ) );

				$processed = $index + 1;
			}

			$current_progress = (int) ( ( $processed / $posts_len ) * 100 );

			if ( $current_progress >= $prev_progress + 5 ) {
				$this->set_progress( $current_progress );

				$prev_progress = $current_progress;
			}
		}

		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			add_action( 'post_updated', 'wp_save_post_revision' );
		}

		$this->set_progress( 100, $processed );
	}

	/**
	 * Performs the adding of the no follow attribute to the links
	 *
	 * @param int $posts_idx
	 *
	 * @return void
	 */
	public function no_follow( int $posts_idx ): void {
		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			remove_action( 'post_updated', 'wp_save_post_revision' );
		}

		if ( $posts_idx === 0 && $this->add_job() < 0 ) {
			return;
		}

		$posts         = $this->get_posts();
		$posts_len     = count( $posts );
		$prev_progress = 1;
		$processed     = 0;

		foreach ( $posts as $index => $post ) {
			if ( $posts_idx > 0 && $index < $posts_idx ) { // Skip already processed items
				continue;
			}

			if ( upd_amazon_memory_time_limit_reached() ) {
				UPD_Amazon_Cron_Jobs::get_instance()->clear( 'upd_amazon_no_follow' );

				UPD_Amazon_Cron_Jobs::get_instance()->schedule_single( time() + 30, 'upd_amazon_no_follow', false, array(
					'posts_idx' => $index,
				) );

				return;
			}

			if ( ( $post->post_content !== "" ) && count( $this->find_links( $post->post_content ) ) > 0 ) {
				$dom = new DOMDocument();
				$dom->loadHTML( $post->post_content );

				foreach ( $dom->getElementsByTagName( 'a' ) as $item ) {
					$link = $item->getAttribute( 'href' );

					if ( ( strpos( $link, '.amazon' ) !== false ) || ( strpos( $link, '.amzn' ) !== false ) || strpos( $link, '//amazon.' ) !== false || strpos( $link, '//amzn.' ) !== false ) {
						$item->setAttribute( 'rel', 'nofollow' );
						$new_content = $dom->saveHTML();
						$post        = array( 'ID' => (int) $post->ID, 'post_content' => $new_content );
						wp_update_post( $post );

						$processed = $index + 1;
					}
				}
			}

			$current_progress = (int) ( ( $processed / $posts_len ) * 100 );

			if ( $current_progress >= $prev_progress + 5 ) {
				$this->set_progress( $current_progress );

				$prev_progress = $current_progress;
			}
		}

		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			add_action( 'post_updated', 'wp_save_post_revision' );
		}

		$this->set_progress( 100, $processed );
	}

	/**
	 * Performs the removing of the Amazon affiliate links
	 *
	 * @param int $posts_idx
	 *
	 * @return void
	 */
	public function no_affiliate( int $posts_idx ): void {
		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			remove_action( 'post_updated', 'wp_save_post_revision' );
		}

		if ( $posts_idx === 0 && $this->add_job() < 0 ) {
			return;
		}

		$posts         = $this->get_posts();
		$posts_len     = count( $posts );
		$prev_progress = 1;
		$processed     = 0;

		foreach ( $posts as $index => $post ) {
			if ( $posts_idx > 0 && $index < $posts_idx ) { // Skip already processed items
				continue;
			}

			if ( upd_amazon_memory_time_limit_reached() ) {
				UPD_Amazon_Cron_Jobs::get_instance()->clear( 'upd_amazon_no_affiliate' );

				UPD_Amazon_Cron_Jobs::get_instance()->schedule_single( time() + 30, 'upd_amazon_no_affiliate', false, array(
					'posts_idx' => $index,
				) );

				return;
			}

			if ( ! empty( $post->post_content ) && count( $this->find_links( $post->post_content ) ) > 0 ) {
				$new_content = preg_replace_callback( '/"(https?):\/\/(www\.)?amzn.to([^"]*)"|"(https?):\/\/(www\.)?amazon.([^"]*)"/', function ( $url ) {
					$newLink = $this->transform_link( str_replace( '"', '', $url[0] ), '' );

					return '"' . $newLink . '"';
				}, $post->post_content );

				$post = array( 'ID' => $post->ID, 'post_content' => $new_content );
				wp_update_post( $post );

				$processed = $index + 1;
			}

			$current_progress = (int) ( ( $processed / $posts_len ) * 100 );

			if ( $current_progress >= $prev_progress + 5 ) {
				$this->set_progress( $current_progress );

				$prev_progress = $current_progress;
			}
		}

		if ( ! defined( 'URALP_USE_REVISIONS' ) ) {
			add_action( 'post_updated', 'wp_save_post_revision' );
		}

		$this->set_progress( 100, $processed );
	}

	/**
	 * Returns the last job
	 *
	 * @param bool $check_finished
	 *
	 * @return array|null
	 */
	public function get_last_job( bool $check_finished = false ): ?array {
		$table = $this->wpdb->prefix . UPD_Amazon_Install::JOBS_TABLE_NAME;
		$job   = $this->wpdb->get_row( "SELECT * FROM $table order by id desc limit 1;", ARRAY_A );

		if ( ! empty( $job ) ) {
			if ( $check_finished && ! empty( $job['finished_at'] ) ) {
				$finished_at          = strtotime( $job['finished_at'] );
				$current_db_timestamp = strtotime( $this->current_timestamp() );

				$time_difference = abs( $current_db_timestamp - $finished_at );

				if ( $time_difference > 15 ) {
					return null;
				}
			}

			return $job;
		}

		return null;
	}

	/**
	 * Adds a new job to the jobs table
	 *
	 * @param string $keyword
	 *
	 * @return int
	 */
	private function add_job( string $keyword = '' ): int {
		$table = $this->wpdb->prefix . UPD_Amazon_Install::JOBS_TABLE_NAME;
		$job   = $this->get_last_job();

		if ( $job !== null && (int) $job['percent'] !== 100 ) {
			return - 1;
		}

		$this->wpdb->insert(
			$table,
			array(
				'keyword' => $keyword,
				'percent' => 0
			)
		);

		return 1;
	}

	/**
	 * Find all Amazon links within a string
	 *
	 * @param string $str
	 *
	 * @return array
	 */
	private function find_links( string $str ): array {
		$pattern = '/"(https?):\/\/(www\.)?amzn.to([^"]*)"|"(https?):\/\/(www\.)?amazon.([^"]*)"/';
		$result  = [];

		preg_match_all( $pattern, $str, $out );

		foreach ( $out[0] as $link ) {
			if ( ( strpos( $link, '.amazon' ) !== false ) || ( strpos( $link, '.amzn' ) !== false )
			     || strpos( $link, '//amazon.' ) !== false || strpos( $link, '//amzn.' ) !== false ) {
				$result[] = $link;
			}
		}

		return $result;
	}

	/**
	 * Trying to get the full Amazon link
	 *
	 * @param string $url
	 *
	 * @return string
	 */
	private function get_full_link( string $url ): string {
		$agent = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322)';
		$args  = array(
			'timeout'     => '10',
			'redirection' => '0',
			'httpversion' => '1.0',
			'blocking'    => true,
			'headers'     => array(),
			'cookies'     => array(),
			'sslverify'   => false,
			'user-agent'  => $agent
		);

		$response = wp_remote_get( $url, $args );

		return wp_remote_retrieve_header( $response, 'location' );
	}

	/**
	 * Transforms Amazon link
	 *
	 * @param string $in_link
	 * @param string $new_val
	 * @param bool $return_full
	 *
	 * @return string
	 */
	private function transform_link( string $in_link, string $new_val, bool $return_full = false ): string {
		$link = $in_link;

		if ( ( $new_val !== '' || $return_full ) && strpos( $link, 'amzn.to' ) !== false ) {
			$location = $this->get_full_link( $link );

			if ( $location ) {
				$link = $location;
			}
		}

		if ( ! $return_full && ( strpos( $link, '.amazon' ) !== false ) ) {
			$url_array = parse_url( trim( $link ) );
			$params    = array();

			parse_str( $url_array['query'], $params );

			if ( $new_val !== '' ) {
				if ( isset( $params['amp;tag'] ) ) {
					$params['amp;tag'] = $new_val;
				} else {
					$params['tag'] = $new_val;
				}
			} else {
				unset( $params['tag'], $params['amp;tag'] );
			}

			$query = '';

			foreach ( $params as $key => $value ) {
				if ( $query !== '' ) {
					$query .= '&';
				}

				$query .= "$key=$value";
			}

			$path = $url_array["path"];

			if ( strpos( $url_array["path"], "tag=" ) !== false ) {
				$path_chunks     = explode( "/", $path );
				$new_path_chunks = [];

				foreach ( $path_chunks as $chunk ) {
					if ( strpos( $chunk, "tag=" ) !== false ) {
						continue;
					}

					$new_path_chunks[] = $chunk;
				}

				$path = implode( "/", $new_path_chunks );
			}

			return $url_array["scheme"] . '://' . $url_array["host"] . $path . "?" . $query;
		}

		return $link;
	}

	/**
	 * Updates the current progress of a job/action
	 *
	 * @param int $progress
	 * @param int|null $processed
	 *
	 * @return void
	 */
	private function set_progress( int $progress, int $processed = null ): void {
		$table = $this->wpdb->prefix . UPD_Amazon_Install::JOBS_TABLE_NAME;
		$job   = $this->get_last_job();

		$data = array(
			'percent' => $progress
		);

		if ( $progress === 100 ) {
			$data['finished_at'] = $this->current_timestamp();

			if ( $processed !== null ) {
				$data['processed'] = $processed;
			}
		}

		$this->wpdb->update(
			$table,
			$data,
			array(
				'id' => $job['id']
			)
		);
	}

	/**
	 * Get all posts
	 *
	 * @return array
	 */
	private function get_posts(): array {
		return get_posts( array(
			'numberposts' => - 1,
			'post_status' => 'any',
			'post_type'   => [ 'page', 'post', 'shortcoder' ],
		) );
	}

	/**
	 * Returns the current timestamp from the database
	 *
	 * @param string $format
	 * @param int|null $day_offset
	 *
	 * @return string
	 */
	private function current_timestamp( string $format = '', int $day_offset = null ): string {
		$current_db_timestamp = $this->wpdb->get_var( 'SELECT CURRENT_TIMESTAMP' );

		if ( ! empty( $format ) ) {
			if ( null !== $day_offset ) {
				$current_db_timestamp .= ' ' . $day_offset . ' day';
			}

			return gmdate( $format, strtotime( $current_db_timestamp ) );
		}

		return $current_db_timestamp;
	}
}
